@@ -10,7 +10,7 @@
 #' @param test_indices Required if \code{data_test} is given. A vector or 1-column data.frame of numeric
 #' row indices or dates (class 'Date' or 'POSIXt') with length \code{nrow(data_test)}.
 #' @param metrics A character vector of common forecast error metrics. The default behavior is to return all metrics.
-#' @param models Optional. A character vector of user-defined model names supplied to \code{train_model()}.
+#' @param models Optional. A character vector of user-defined model names supplied to \code{train_model()} to filter results.
 #' @param horizons Optional. A numeric vector to filter results by horizon.
 #' @param windows Optional. A numeric vector to filter results by validation window number.
 #' @param group_filter Optional. A string for filtering plot results for grouped time-series
@@ -19,7 +19,7 @@
 #' @return An S3 object of class 'validation_error' or 'forecast_error': A list of data.frames
 #' of error metrics for the validation datasets or forecast dataset depending
 #' on the \code{data_test} argument. An input to \code{data_results} from \code{combine_forecasts()} will return
-#' a single data.frame with results for each model passed in \code{...}. \cr
+#' a single data.frame with results for each model passed in \code{combine_forecasts(...)}. \cr
 #'
 #' A list containing: \cr
 #'
@@ -31,7 +31,7 @@
 #' @section Error Metrics:
 #'
 #' \itemize{
-#'   \item \code{mae}: Mean absolute error
+#'   \item \code{mae}: Mean absolute error (works with factor outcomes)
 #'   \item \code{mape}: Mean absolute percentage error
 #'   \item \code{mdape}: Median absolute percentage error
 #'   \item \code{smape}: Symmetrical mean absolute percentage error
@@ -83,14 +83,15 @@ return_error <- function(data_results, data_test = NULL, test_indices = NULL,
   data <- data_results
   rm(data_results)
 
-  outcome_names <- attributes(data)$outcome_names
+  method <- attributes(data)$method
+  outcome_name <- attributes(data)$outcome_name
   outcome_levels <- attributes(data)$outcome_levels
   groups <- attributes(data)$groups
   #----------------------------------------------------------------------------
-  # For factor outcomes, is the prediction a factor level or probability.
+  # For factor outcomes, is the prediction a factor level or probability?
   if (!is.null(outcome_levels)) {
 
-    factor_level <- if (any(names(data) %in% paste0(outcome_names, "_pred"))) {TRUE} else {FALSE}
+    factor_level <- if (any(names(data) %in% paste0(outcome_name, "_pred"))) {TRUE} else {FALSE}
     factor_prob <- !factor_level
 
     if (!all(metrics %in% c("mae"))) {
@@ -113,29 +114,37 @@ return_error <- function(data_results, data_test = NULL, test_indices = NULL,
 
     data_test$forecast_period <- test_indices
 
-    data_test <- dplyr::select(data_test, .data$forecast_period, outcome_names, !!groups)
+    data_test <- dplyr::select(data_test, .data$forecast_period, !!outcome_name, !!groups)
+
+    data$model_forecast_horizon <- data$horizon  # Added to use the same dplyr code as direct forecasting.
 
     data <- dplyr::inner_join(data, data_test, by = c("forecast_period", groups))
+
+    if (nrow(data) == 0) {
+      stop("The test dataset in 'data_test' does not overlap with the forecast period in 'data_results'.")
+    }
   }
 
   if (methods::is(data, "training_results")) {
     # Change the forecast horizon name to "horizon", which, although it makes sense from a naming perspective,
     # will reduce the amount of code below.
-    data$horizon <- data$model_forecast_horizon
-    data$model_forecast_horizon <- NULL
+    if (method == "direct") {
+      data$horizon <- data$model_forecast_horizon
+      data$model_forecast_horizon <- NULL
+    }
   }
   #----------------------------------------------------------------------------
   # Residual calculations
   if (is.null(outcome_levels)) {  # Numeric outcome.
 
-    data$residual <- data[, outcome_names] - data[, paste0(outcome_names, "_pred")]
+    data$residual <- data[, outcome_name] - data[, paste0(outcome_name, "_pred")]
 
   } else {  # Factor outcome.
 
     if (factor_level) {
 
       # Binary accuracy/residual. A residual of 1 is an incorrect classification.
-      data$residual <- ifelse(data[, outcome_names] != data[, paste0(outcome_names, "_pred")], 1, 0)
+      data$residual <- ifelse(as.character(data[, outcome_name, drop = TRUE]) != as.character(data[, paste0(outcome_name, "_pred"), drop = TRUE]), 1, 0)
 
     } else {  # Class probabilities were predicted.
 
@@ -174,7 +183,7 @@ return_error <- function(data_results, data_test = NULL, test_indices = NULL,
 
     # Compute error metrics at the validation window level.
     data_1 <- data %>%
-      dplyr::group_by_at(dplyr::vars(.data$model, .data$horizon, .data$window_number, !!groups)) %>%
+      dplyr::group_by_at(dplyr::vars(.data$model, .data$horizon, .data$window_number)) %>%
       dplyr::mutate("window_start" = min(.data$valid_indices, na.rm = TRUE),
                     "window_stop" = max(.data$valid_indices, na.rm = TRUE),
                     "window_midpoint" = base::mean(.data$valid_indices, na.rm = TRUE)) %>%
@@ -184,33 +193,33 @@ return_error <- function(data_results, data_test = NULL, test_indices = NULL,
       dplyr::summarize_at(dplyr::vars(1),  # 1 is a col position that gets the fun to run; args x, y, and z defined below.
                           .funs = error_functions,
                           x = rlang::quo(.data$residual),
-                          y = rlang::sym(outcome_names),
-                          z = rlang::sym(paste0(outcome_names, "_pred"))
+                          y = rlang::sym(outcome_name),
+                          z = rlang::sym(paste0(outcome_name, "_pred"))
                           )
 
     # Compute error metric by horizon and window length across all validation windows.
     data_2 <- data %>%
-      dplyr::group_by_at(dplyr::vars(.data$model, .data$horizon, !!groups)) %>%
+      dplyr::group_by_at(dplyr::vars(.data$model, .data$horizon)) %>%
       dplyr::mutate("window_start" = min(.data$valid_indices, na.rm = TRUE),
                     "window_stop" = max(.data$valid_indices, na.rm = TRUE)) %>%
       dplyr::group_by_at(dplyr::vars(.data$model, .data$horizon, !!groups, .data$window_start, .data$window_stop)) %>%
       dplyr::summarize_at(dplyr::vars(1),  # 1 is a col position that gets the fun to run; args x, y, and z defined below.
                           .funs = error_functions,
                           x = rlang::quo(.data$residual),
-                          y = rlang::sym(outcome_names),
-                          z = rlang::sym(paste0(outcome_names, "_pred")))
+                          y = rlang::sym(outcome_name),
+                          z = rlang::sym(paste0(outcome_name, "_pred")))
 
     # Compute error metric by model.
     data_3 <- data %>%
-      dplyr::group_by_at(dplyr::vars(.data$model, !!groups)) %>%
+      dplyr::group_by_at(dplyr::vars(.data$model)) %>%
       dplyr::mutate("window_start" = min(.data$valid_indices, na.rm = TRUE),
                     "window_stop" = max(.data$valid_indices, na.rm = TRUE)) %>%
       dplyr::group_by_at(dplyr::vars(.data$model, !!groups, .data$window_start, .data$window_stop)) %>%
       dplyr::summarize_at(dplyr::vars(1),  # 1 is a col position that gets the fun to run; args x, y, and z defined below.
                           .funs = error_functions,
                           x = rlang::quo(.data$residual),
-                          y = rlang::sym(outcome_names),
-                          z = rlang::sym(paste0(outcome_names, "_pred")))
+                          y = rlang::sym(outcome_name),
+                          z = rlang::sym(paste0(outcome_name, "_pred")))
     #--------------------------------------------------------------------------
     } else if (!is_forecastML) {  # Error metrics for the forecast_results class which has no validation windows and slightly different grouping columns.
 
@@ -222,8 +231,8 @@ return_error <- function(data_results, data_test = NULL, test_indices = NULL,
         dplyr::summarize_at(dplyr::vars(1),  # 1 is a col position that gets the fun to run; args x, y, and z defined below.
                             .funs = error_functions,
                             x = rlang::quo(.data$residual),
-                            y = rlang::sym(outcome_names),
-                            z = rlang::sym(paste0(outcome_names, "_pred")))
+                            y = rlang::sym(outcome_name),
+                            z = rlang::sym(paste0(outcome_name, "_pred")))
 
       # Compute error metric by model.
       data_3 <- data %>%
@@ -231,8 +240,8 @@ return_error <- function(data_results, data_test = NULL, test_indices = NULL,
         dplyr::summarize_at(dplyr::vars(1),  # 1 is a col position that gets the fun to run; args x, y, and z defined below.
                             .funs = error_functions,
                             x = rlang::quo(.data$residual),
-                            y = rlang::sym(outcome_names),
-                            z = rlang::sym(paste0(outcome_names, "_pred")))
+                            y = rlang::sym(outcome_name),
+                            z = rlang::sym(paste0(outcome_name, "_pred")))
 
     } else {  # Final forecasts from combine_forecasts().
 
@@ -241,8 +250,8 @@ return_error <- function(data_results, data_test = NULL, test_indices = NULL,
         dplyr::summarize_at(dplyr::vars(1),  # 1 is a col position that gets the fun to run; args x, y, and z defined below.
                             .funs = error_functions,
                             x = rlang::quo(.data$residual),
-                            y = rlang::sym(outcome_names),
-                            z = rlang::sym(paste0(outcome_names, "_pred")))
+                            y = rlang::sym(outcome_name),
+                            z = rlang::sym(paste0(outcome_name, "_pred")))
 
     }  # End error metrics for forecast results.
   #----------------------------------------------------------------------------
@@ -282,49 +291,100 @@ return_error <- function(data_results, data_test = NULL, test_indices = NULL,
 #' @param x An object of class 'validation_error' from \code{return_error()}.
 #' @param data_results An object of class 'training_results' from \code{predict.forecast_model()}.
 #' @param type Select plot type; \code{type = "time"} is the default plot.
+#' @param metric Select error metric to plot (e.g., "mae"); \code{attributes(x)$error_metrics[1]} is the default metric.
 #' @param models Optional. A vector of user-defined model names from \code{train_model()} to filter results.
 #' @param horizons Optional. A numeric vector to filter results by horizon.
 #' @param windows Optional. A numeric vector to filter results by validation window number.
 #' @param group_filter A string for filtering plot results for grouped time-series (e.g., \code{"group_col_1 == 'A'"}).
+#' @param facet Optional. A formula with any combination of \code{horizon}, \code{model}, or \code{group} (for grouped time series).
+#' passed to \code{ggplot2::facet_grid()} internally (e.g., \code{horizon ~ model}, \code{horizon + model ~ .}, \code{~ horizon + group}).
+#' Can be \code{NULL}. The default faceting is set internally depending on the plot \code{type}.
 #' @param ... Not used.
 #' @return Forecast error plots of class 'ggplot'.
 #' @export
-plot.validation_error <- function(x, data_results, type = c("time", "horizon", "global"),
-                                  models = NULL, horizons = NULL, windows = NULL, group_filter = NULL, ...) { # nocov start
-
-  if(!methods::is(x, "validation_error")) {
-    stop("The 'x' argument takes an object of class 'validation_error' as input. Run return_error() first.")
-  }
+plot.validation_error <- function(x, data_results, type = c("time", "horizon", "global"), metric = NULL,
+                                  facet = NULL, models = NULL, horizons = NULL, windows = NULL, group_filter = NULL, ...) { # nocov start
 
+  #----------------------------------------------------------------------------
   data_error <- x
 
-  if(!methods::is(data_results, "training_results")) {
+  if (!methods::is(data_results, "training_results")) {
     stop("The 'data_results' argument takes an object of class 'training_results' as input. Run predict.forecast_model() first.")
   }
 
   type <- type[1]
 
-  outcome_names <- attributes(data_results)$outcome_names
+  method <- attributes(data_results)$method
+  outcome_name <- attributes(data_results)$outcome_name
   groups <- attributes(data_results)$groups
 
-  error_metrics <- attributes(data_error)$error_metrics
+  error_metrics <- metric
 
-  data_plot <- data_error$error_by_window
+  if (is.null(error_metrics)) {
+    error_metrics <- attributes(data_error)$error_metrics[1]
+  }
 
-  models <- if (is.null(models)) {unique(data_results$model)} else {models}
-  horizons <- if (is.null(horizons)) {unique(data_results$model_forecast_horizon)} else {horizons}
-  windows <- if (is.null(windows)) {unique(data_results$window_number)} else {windows}
+  if (!error_metrics %in% attributes(data_error)$error_metrics) {
+    stop("The error metric in 'metric' is not in the validation error dataset 'x'; re-run return_error() with this metric.")
+  }
+  #----------------------------------------------------------------------------
+  # Set default plot facets for each plot type.
+  if (is.null(facet)) {
 
+    if (type == "time") {
+
+      facet <- ~ horizon ~ model
+
+    } else if (type == "horizon") {
+
+      facet <- horizon ~ model
+
+    } else if (type == "global") {
+
+      facet <- horizon ~ model
+    }
+  }
+
+  facets <- forecastML_facet_plot(facet, groups)  # Function in zzz.R.
+  facet <- facets[[1]]
+  facet_names <- facets[[2]]
+
+  if (type == "time") {
+
+    data_plot <- data_error$error_by_window
+
+  } else if (type == "horizon") {
+
+    data_plot <- data_error$error_by_horizon
+    data_plot$window_number <- TRUE  # Not in data, added for filtering.
+
+  } else if (type == "global") {
+
+    data_plot <- data_error$error_global
+    data_plot$window_number <- TRUE  # Not in data, added for filtering.
+    data_plot$horizon <- TRUE  # Not in data, added for filtering.
+  }
   #----------------------------------------------------------------------------
   # Filter the datasets based on user input.
-  data_plot <- data_plot[data_plot$horizon %in% horizons &
-                         data_plot$model %in% models & data_plot$window_number %in% windows, ]
+  models <- if (is.null(models)) {unique(data_results$model)} else {models}
+  horizons <- if (is.null(horizons)) {
+    if (method == "direct") {
+      unique(data_results$model_forecast_horizon)
+    } else if (method == "multi_output") {
+      unique(data_results$horizon)
+    }} else {
+      horizons
+      }
+  windows <- if (is.null(windows)) {unique(data_results$window_number)} else {windows}
 
-  data_results$horizon <- data_results$model_forecast_horizon
-  data_results$model_forecast_horizon <- NULL
+  data_plot <- data_plot[data_plot$horizon %in% horizons & data_plot$model %in% models & data_plot$window_number %in% windows, ]
 
-  data_results <- data_results[data_results$horizon %in% horizons &
-                               data_results$model %in% models & data_results$window_number %in% windows, ]
+  if (method == "direct") {
+    data_results$horizon <- data_results$model_forecast_horizon
+    data_results$model_forecast_horizon <- NULL
+  }
+
+  data_results <- data_results[data_results$horizon %in% horizons & data_results$model %in% models & data_results$window_number %in% windows, ]
   #----------------------------------------------------------------------------
   # User filtering to display select results in grouped time-series.
   if (!is.null(group_filter)) {
@@ -333,135 +393,105 @@ plot.validation_error <- function(x, data_results, type = c("time", "horizon", "
   }
   #----------------------------------------------------------------------------
   # Melt the data for plotting.
-  data_plot <- tidyr::gather(data_plot, "error_metric", "value",
-                             -!!names(data_plot)[!names(data_plot) %in% error_metrics])
+  data_plot <- tidyr::gather(data_plot, "error_metric", "value", -!!names(data_plot)[!names(data_plot) %in% error_metrics])
+  #--------------------------------------------------------------------------
+  # ggplot colors and facets are complimentary: all facets, same color; all colors, no facet.
+  ggplot_color <- c(c("model", "horizon", groups)[!c("model", "horizon", groups) %in% facet_names])
+  #--------------------------------------------------------------------------
+
+  if (is.null(groups)) {
+    data_plot <- dplyr::arrange(data_plot, .data$model, .data$horizon, .data$window_number)
+  } else {
+    data_plot <- dplyr::arrange(data_plot, .data$model, .data$horizon, .data$window_number, !!rlang::sym(groups))
+  }
+
+  if (type == "global") {
+    data_plot$horizon <- "All"
+    data_plot$horizon <- ordered(data_plot$horizon)
+  }
+
+  data_plot$horizon <- factor(data_plot$horizon, levels = unique(data_plot$horizon), ordered = TRUE)
+  data_plot[, groups] <- factor(data_plot[, groups], levels = unique(data_plot[, groups]), ordered = TRUE)
+
+  data_plot$ggplot_color <- apply(data_plot[,  ggplot_color, drop = FALSE], 1, function(x) {paste(x, collapse = "-")})
+
+  data_plot$ggplot_color <- factor(data_plot$ggplot_color, levels = unique(data_plot$ggplot_color), ordered = TRUE)
+
+  # Give predictions a name in the legend if plot is faceted by model and horizon (and group if groups are given).
+  if (length(ggplot_color) == 0) {
+    data_plot$ggplot_color <- "Error"
+    data_plot$ggplot_color <- ordered(data_plot$ggplot_color)
+  }
+
+  temp_1 <- unlist(Map(function(x) {toupper(substr(x[1], 1, 1))}, ggplot_color))
+  temp_2 <- unlist(Map(function(x) {substr(x, 2, nchar(x))}, ggplot_color))
+  x_axis_title <- paste(temp_1, temp_2, sep = "")
+  x_axis_title <- paste(x_axis_title, collapse = " + ")
 
   if (type == "time") {
 
-    data_plot$ggplot_color_group <- apply(data_plot[, c("model", groups), drop = FALSE], 1, paste, collapse = "-")
-    data_plot$ggplot_color_group <- ordered(data_plot$ggplot_color_group)
-
     p <- ggplot()
+
     if (length(unique(data_plot$window_midpoint)) != 1) {
+
       p <- p + geom_line(data = data_plot, aes(x = .data$window_midpoint,
                                                y = .data$value,
-                                               color = .data$ggplot_color_group,
-                                               group = .data$ggplot_color_group), size = 1.05)
+                                               color = .data$ggplot_color,
+                                               group = .data$ggplot_color
+                                               ), size = 1.05)
     }
+
     p <- p + geom_point(data = data_plot, aes(x = .data$window_midpoint,
                                               y = .data$value,
-                                              color = .data$ggplot_color_group,
-                                              group = .data$ggplot_color_group), show.legend = FALSE)
+                                              color = .data$ggplot_color,
+                                              group = .data$ggplot_color
+                                              ))
     p <- p + scale_color_viridis_d()
-    p <- p + facet_grid(error_metric ~ horizon, scales = "free")
+    p <- p + facet_grid(facet, scales = "free")
     p <- p + theme_bw()
-    p <- p + xlab("Dataset index") + ylab("Forecast error metric") + labs(color = "Model") +
-      ggtitle("Forecast Error by Validation Window - Faceted by horizon and metric")
-    return(p)
+    p <- p + xlab("Dataset index") + ylab(paste0("Forecast error metric (", error_metrics, ")")) + labs(color = x_axis_title) +
+      ggtitle("Forecast Error by Validation Window")
+
+    return(suppressWarnings(p))
   }
 
   if (type == "horizon") {
 
-    data_plot <- data_error$error_by_horizon
-
-    models <- if (is.null(models)) {unique(data_plot$model)} else {models}
-    horizons <- if (is.null(horizons)) {unique(data_plot$horizon)} else {horizons}
-
-    data_plot <- data_plot[data_plot$horizon %in% horizons &
-                                           data_plot$model %in% models, ]
-
-    if (!is.null(group_filter)) {
-      data_plot <- dplyr::filter(data_plot, eval(parse(text = group_filter)))
-    }
-
-    # Melt the data for plotting.
-    data_plot <- tidyr::gather(data_plot, "error_metric", "value",
-                                       -!!names(data_plot)[!names(data_plot) %in% error_metrics])
-
-    data_plot$ggplot_color_group <- apply(data_plot[, c("model", groups), drop = FALSE], 1, paste, collapse = "-")
-    data_plot$ggplot_color_group <- ordered(data_plot$ggplot_color_group)
-
-    data_plot$ggplot_label_group <- apply(data_plot[, c("model", "horizon", groups), drop = FALSE], 1, paste, collapse = "-")
-
     p <- ggplot()
-    p <- p + geom_point(data = data_plot,
-                        aes(x = ordered(.data$horizon),
+    p <- p + geom_col(data = data_plot,
+                        aes(x = .data$ggplot_color,
                             y = .data$value,
-                            color = .data$ggplot_color_group,
-                            group = .data$ggplot_color_group), size = 1.05)
-
-    if (length(unique(data_plot$horizon)) != 1) {
-        p <- p + geom_line(data = data_plot,
-                           aes(x = ordered(.data$horizon),
-                               y = .data$value,
-                               color = .data$ggplot_color_group,
-                               group = .data$ggplot_color_group),
-                           size = 1.05, alpha = .20, show.legend = FALSE)
-    }
-
-    p <- p + geom_label(data = data_plot,
-                        aes(x = ordered(.data$horizon),
-                            y = .data$value,
-                            color = .data$ggplot_color_group,
-                            group = .data$ggplot_label_group,
-                            label = round(.data$value, 1)),
-                        position = position_dodge(.5), show.legend = FALSE)
-
+                            fill = .data$ggplot_color,
+                            group = .data$ggplot_color))
+    p <- p + facet_grid(facet, scales = "free_y")
     p <- p + scale_color_viridis_d()
-    p <- p + facet_grid(error_metric ~ ., scales = "free")
-    p <- p + theme_bw()
-    p <- p + xlab("Forecast horizon") + ylab("Forecast error metric") + labs(color = "Model", alpha = NULL) +
-      ggtitle("Forecast Error by Forecast Horizon - Faceted by metric")
-    return(p)
+    p <- p + theme_bw() + theme(
+      axis.text.x = element_blank()
+    )
+    p <- p + xlab(x_axis_title) + ylab(paste0("Forecast error metric (", error_metrics, ")")) + labs(fill = x_axis_title, alpha = NULL) +
+      ggtitle("Forecast Error by Forecast Horizon")
+    p
+
+    return(suppressWarnings(p))
   }
 
   if (type == "global") {
 
-    data_plot <- data_error$error_global
+    p <- ggplot()
 
-    data_plot <- data_plot[data_plot$model %in% models, ]
-
-    if (!is.null(group_filter)) {
-      data_plot <- dplyr::filter(data_plot, eval(parse(text = group_filter)))
-    }
-
-    data_plot$ggplot_color_group <- apply(data_plot[, c("model", groups), drop = FALSE], 1, paste, collapse = "-")
-    data_plot$ggplot_color_group <- ordered(data_plot$ggplot_color_group)
-
-    # If groups exist, fill the bars the same color by groups across models; otherwise, fill the bars by model.
-    if (!is.null(groups)) {
-
-      data_plot$ggplot_fill_group <- apply(data_plot[, c(groups), drop = FALSE], 1, paste, collapse = "-")
-      data_plot$ggplot_fill_group <- factor(data_plot$ggplot_fill_group, levels = unique(data_plot$ggplot_fill_group),
-                                            ordered = TRUE)
-
-    } else {
-
-      data_plot$ggplot_fill_group <- apply(data_plot[, c("model", groups), drop = FALSE], 1, paste, collapse = "-")
-      data_plot$ggplot_fill_group <- ordered(data_plot$ggplot_fill_group)
-    }
-
-    data_plot <- tidyr::gather(data_plot, "error_metric", "value",
-                                       -!!names(data_plot)[!names(data_plot) %in% error_metrics])
-
-    p <- ggplot(data_plot)
     p <- p + geom_bar(data = data_plot,
-                      aes(x = .data$model,
+                      aes(x = .data$ggplot_color,
                           y = .data$value,
-                          fill = .data$ggplot_fill_group,
-                          group = .data$ggplot_color_group),
+                          fill = .data$ggplot_color),
                       stat = "identity", position = position_dodge(width = 1))
-    p <- p + geom_label(data = data_plot,
-                        aes(x = .data$model,
-                            y = .data$value,
-                            label = round(.data$value, 1),
-                            group = .data$ggplot_color_group),
-                        show.legend = FALSE, position = position_dodge(width = 1))
     p <- p + scale_fill_viridis_d()
-    p <- p + facet_grid(.data$error_metric ~ ., scales = "free")
-    p <- p + theme_bw()
-    p <- p + xlab("") + ylab("Forecast error metric") + labs(fill = "", alpha = NULL) +
-      ggtitle("Forecast Error - Faceted by metric")
-    return(p)
+    p <- p + facet_grid(facet, scales = "free")
+    p <- p + theme_bw() + theme(
+      axis.text.x = element_blank()
+    )
+    p <- p + xlab(x_axis_title) + ylab(paste0("Forecast error metric (", error_metrics, ")")) + labs(fill = x_axis_title, alpha = NULL) +
+      ggtitle("Forecast Error Across Validation Windows and Horizons")
+
+    return(suppressWarnings(p))
   }
 } # nocov end
